package de.unibi.airobots.bonsai.actuators;

import java.io.IOException;

import de.unibi.airobots.bonsai.core.Actuator;

/**
 * Interface for arena entering and localization actuators.
 * 
 * @author ltwardon
 */
public interface EnterArenaAndLocalizeActuator extends Actuator {

    /**
     * Let the robot enter the arena and do a mini-exploration in order to localize itself on the map
     * 
     * @param meters
     *      How far the robot should enter the arena
     * 
     * @throws IOException
     *             communication error
     */
    void enterArenaAndLocalize(double meters) throws IOException;
    
    /**
     * Let the robot enter the arena and do a mini-exploration in order to localize itself on the map
     * 
     * @param meters
     *      How far the robot should enter the arena
     * @param priorKnowledgeX
     *      Prior Knowledge about where the robot might be when starting to localize itself (X-Coord)
     * @param priorKnowledgeY
     *      Prior Knowledge about where the robot might be when starting to localize itself (Y-Coord)
     * @param priorKnowledgeDiv
     *      Prior Knowledge about where the robot might be when starting to localize itself (Divergence from X, Y)
     * 
     * @throws IOException
     *             communication error
     */
    void enterArenaAndLocalize(double meters, double priorKnowledgeX, double priorKnowledgeY, double priorKnowledgeDiv) throws IOException;

    /**
     * Wait until the robot has entered the arena and finished the exploration/localization
     * 
     * @throws IOException
     *             communication error
     */
    void waitUntilLocalizationFinished() throws IOException;
    
}
