package de.unibi.airobots.bonsai.actuators;

import java.io.IOException;

import net.sf.xcf.MethodNotFoundException;
import net.sf.xcf.UserException;
import de.unibi.airobots.bonsai.actuators.data.CameraPosition;
import de.unibi.airobots.bonsai.core.Actuator;

/**
 * Old interface to control a pan-tilt camera.
 * 
 * @author jwienke
 */
public interface OldCameraActuator extends Actuator {

    /**
     * Set the position of the Camera.
     * 
     * @param pan
     *            angle in radiant (0=forward, angles CCW)
     * @param tilt
     *            angle in radiant (0=forward, negative = down)
     * @throws IOException
     */
    void setPos(double pan, double tilt) throws IOException;

    /**
     * set the Zoom of the Camera.
     * 
     * @param zoom
     *            of the camera between 0 and 1 (0=wide, 1=tele)
     * @throws IOException
     */
    void setZoom(double zoom) throws IOException;

    /**
     * set the speed of pan movements.
     * 
     * @param panspeed
     *            of the cameras pan movement
     * @throws IOException
     */
    void setPanSpeed(double panspeed) throws IOException;

    /**
     * set the speed of tilt movements.
     * 
     * @param tiltspeed
     *            of the cameras tilt movement
     * @throws IOException
     */
    void setTiltSpeed(double tiltspeed) throws IOException;

    /**
     * set the zoom speed.
     * 
     * @param zoomspeed
     *            speed
     * @throws IOException
     */
    void setZoomSpeed(double zoomspeed) throws IOException;

    /**
     * Get the current zoom of the camera.
     * 
     * @return [0..1] 0 = wide, 1 = tele
     * @throws IOException
     *             communication error
     */
    float getZoomPos() throws IOException;

    CameraPosition getCameraPosition() throws IOException,
            MethodNotFoundException, UserException;

}
