package de.unibi.airobots.bonsai.actuators;

import java.io.IOException;

import de.unibi.airobots.bonsai.core.Actuator;

/**
 * Interface for actuators controlling a speech synthesizer.
 * 
 * @author jwienke
 * @author arothert
 */
public interface SpeechActuator extends Actuator {

    /**
     * Say method with the ability to chose between synchronous and asynchronous
     * speech synthesis.
     * 
     * @param text
     *            text to say
     * @param async
     *            if <code>true</code>, wait until the complete text was said,
     *            else return as fast as possible from this method call
     * @throws IOException
     *             communication error
     */
    void say(String text, boolean async) throws IOException;

    /**
     * Synchronous say method. Returns after the complete text was said.
     * 
     * @param text
     *            text to say
     * @param setDefault_accent
     *          if true default accentuation is used, else not.
     * @throws IOException
     *             communication error
     */
    void say(String text) throws IOException;
    
    /**
     * For self-written prosody for the whole text.
     * @param accented_text
     * @throws IOException
     */
    void sayAccentuated(String accented_text) throws IOException;
    
    /**
     * For normal text, maybe accentuations within the text. 
     * Additionally a configuration file path to set the overall prosody of a text must be given.
     * @param accented_text
     * @param prosodyConfig
     * @throws IOException
     */
    void sayAccentuated(String accented_text, String prosodyConfig) throws IOException;
    
    /**
     * For self-written prosody for the whole text.
     * @param accented_text
     *          text to say, might invole specific accentuations of parts 
     *          of the text (MARYXML - prosody - style).
     * @param async
     *          if <code>true</code>, wait until the complete text was said,
     *          else return as fast as possible from this method call
     * @throws IOException
     */
    void sayAccentuated(String accented_text, boolean async) throws IOException;
    
    /**
     * For normal text, maybe accentuations within the text. 
     * Additionally a configuration file path to set the overall prosody of a text must be given.
     * 
     * @param accented_text
     *          text to say, might invole specific accentuations of parts 
     *          of the text (MARYXML - prosody - style).
     * @param async 
     *          if <code>true</code>, wait until the complete text was said,
     *          else return as fast as possible from this method call
     * @param prosodyConfig
     *          path and file for overall prosody setting of the accented_text
     * @throws IOException
     */
    void sayAccentuated(String accented_text, boolean async, String prosodyConfig) throws IOException;

}
