package de.unibi.airobots.bonsai.core.configuration;

import java.util.Map;

/**
 * Basic configuration result returned by configuration parsers.
 * 
 * @author jwienke
 */
public abstract class ConfiguredCoreObject {

    private String key;
    private Map<String, String> options;
    private String factoryClassName;

    /**
     * Constructor.
     */
    public ConfiguredCoreObject() {
        super();
    }

    /**
     * Returns the key specified in the configuration.
     * 
     * @return key
     */
    public String getKey() {
        return key;
    }

    /**
     * Sets the key specified in the configuration.
     * 
     * @param key
     *            key
     */
    public void setKey(String key) {
        this.key = key;
    }

    /**
     * Returns the key-value style options for this core object specified in the
     * config.
     * 
     * @return options
     */
    public Map<String, String> getOptions() {
        return options;
    }

    /**
     * Sets the key-value style options for this core object specified in the
     * config.
     * 
     * @param options
     *            options
     */
    public void setOptions(Map<String, String> options) {
        this.options = options;
    }

    /**
     * Returns the fully specified class name of the factory that shall produce
     * this core object as specified in the config.
     * 
     * @return factory class name
     */
    public String getFactoryClassName() {
        return factoryClassName;
    }

    /**
     * Stes the fully specified class name of the factory that shall produce
     * this core object as specified in the config.
     * 
     * @param factoryClassName
     *            factory class name
     */
    public void setFactoryClassName(String factoryClassName) {
        this.factoryClassName = factoryClassName;
    }

}
