package de.unibi.airobots.bonsai.examples;

import java.awt.Color;
import java.awt.Component;
import java.awt.Container;
import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.IOException;
import java.net.URISyntaxException;
import java.text.DecimalFormat;
import java.util.LinkedList;
import java.util.Vector;

import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JSlider;
import javax.swing.JTextField;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

import org.apache.log4j.BasicConfigurator;
import org.apache.log4j.Logger;

import de.unibi.airobots.bonsai.actuators.PoseActuatorTobi;
import de.unibi.airobots.bonsai.core.BonsaiManager;
import de.unibi.airobots.bonsai.core.BonsaiManager.ConfigurationException;

public class ArmControlRemote extends JPanel implements ActionListener {

    private static final long serialVersionUID = 6130916583873970839L;
    private static final int sliderMinValue0 = -100;
    private static final int sliderMaxValue0 = 100;
    private static final int sliderMinValue1 = -150;
    private static final int sliderMaxValue1 = 240;
    private static final int sliderMinValue2 = -219;
    private static final int sliderMaxValue2 = 200;
    private static final int sliderMinValue3 = -200;
    private static final int sliderMaxValue3 = 200;
    private static final int sliderMinValue4 = -200;
    private static final int sliderMaxValue4 = 200;
    private static final int sliderMinValue5 = -250;
    private static final int sliderMaxValue5 = 250;
    private static final int sliderStartValue = 0;

    private static final DecimalFormat f = new DecimalFormat("#0.0000");

    private final static Logger logger =
            Logger.getLogger(ArmControlRemote.class);

    private PoseActuatorTobi poseActuatorTobi;

    private Vector<String> movementsList;

    private JSlider joint0Slider;
    private JTextField joint0;
    private JSlider joint1Slider;
    private JTextField joint1;
    private JSlider joint2Slider;
    private JTextField joint2;
    private JSlider joint3Slider;
    private JTextField joint3;
    private JSlider joint4Slider;
    private JTextField joint4;
    private JSlider joint5Slider;
    private JTextField joint5;
    private JTextField gotopx;
    private JTextField gotopy;
    private JTextField gotopz;
    private JTextField gotonx;
    private JTextField gotony;
    private JTextField gotonz;
    private JTextField gotoox;
    private JTextField gotooy;
    private JTextField gotooz;
    private JTextField gotoax;
    private JTextField gotoay;
    private JTextField gotoaz;
    private JTextField setMovementsTxt;
    private JComboBox movementBox;
    private JLabel itWorks;

    public ArmControlRemote() {
        BasicConfigurator.configure();
        logger.info("... PoseActuator ...");

        if (BasicRemote.class.getResource("basicremote.xml") != null) {
            try {
                BonsaiManager.getInstance().configure(
                        BasicRemote.class.getResource("basicremote.xml")
                                .toURI());
            } catch (IllegalStateException e) {
                logger.error(e);
            } catch (ConfigurationException e) {
                logger.error(e);
            } catch (URISyntaxException e) {
                logger.error(e);
            }
        } else {
            BonsaiManager.getInstance().configure(
                    "/vol/robocup/trunk/etc/bonsai_config/basicremote.xml");
        }

        poseActuatorTobi =
                BonsaiManager.getInstance().createActuator("PoseActuatorTobi",
                        PoseActuatorTobi.class);

        try {
            movementsList = poseActuatorTobi.listMovements();
            System.out.println(movementsList);
        } catch (IOException e) {
            logger.error("IOException:" + e);
        }
        guiSetup();
    }

    /**
     * A useful function to add all components into the GridBagLayout
     */
    private static void addComponent(Container cont, GridBagLayout gbl,
            Component c, int x, int y, int width, int height, double weightx,
            double weighty) {
        GridBagConstraints gbc = new GridBagConstraints();
        gbc.fill = GridBagConstraints.BOTH;
        gbc.gridx = x;
        gbc.gridy = y;
        gbc.gridwidth = width;
        gbc.gridheight = height;
        gbc.weightx = weightx;
        gbc.weighty = weighty;
        gbl.setConstraints(c, gbc);
        cont.add(c);
      
    }

    /**
     * Containing everything concerning the GUI.
     */
    private void guiSetup() {
        JFrame gui = new JFrame("ArmControl Remote");
        gui.setDefaultCloseOperation(JFrame.DISPOSE_ON_CLOSE);
        gui.setLocation(100, 200);

        Container pane = gui.getContentPane();
        GridBagLayout gbl = new GridBagLayout();
        pane.setLayout(gbl);

        JTextField text1Field = new JTextField();
        text1Field.setPreferredSize(new Dimension(200, 200));

        joint0 = new JTextField("", 7);
        joint0.setEnabled(false);
        joint0.setDisabledTextColor(new Color(000000));
        joint0Slider =
                new JSlider(JSlider.HORIZONTAL, sliderMinValue0,
                        sliderMaxValue0, sliderStartValue);
        joint0Slider.addChangeListener(new ChangeListener() {

            public void stateChanged(ChangeEvent e) {
                joint0
                        .setText(f
                                .format((double) joint0Slider.getValue() / 100));
            }
        });

        joint1 = new JTextField("", 7);
        joint1.setEnabled(false);
        joint1.setDisabledTextColor(new Color(000000));
        joint1Slider =
                new JSlider(JSlider.HORIZONTAL, sliderMinValue1,
                        sliderMaxValue1, sliderStartValue);
        joint1Slider.addChangeListener(new ChangeListener() {

            public void stateChanged(ChangeEvent e) {
                joint1
                        .setText(f
                                .format((double) joint1Slider.getValue() / 100));
            }
        });

        joint2 = new JTextField("", 7);
        joint2.setEnabled(false);
        joint2.setDisabledTextColor(new Color(000000));
        joint2Slider =
                new JSlider(JSlider.HORIZONTAL, sliderMinValue2,
                        sliderMaxValue2, sliderStartValue);
        joint2Slider.addChangeListener(new ChangeListener() {

            public void stateChanged(ChangeEvent e) {
                joint2
                        .setText(f
                                .format((double) joint2Slider.getValue() / 100));
            }
        });

        joint3 = new JTextField("", 7);
        joint3.setEnabled(false);
        joint3.setDisabledTextColor(new Color(000000));
        joint3Slider =
                new JSlider(JSlider.HORIZONTAL, sliderMinValue3,
                        sliderMaxValue3, sliderStartValue);
        joint3Slider.addChangeListener(new ChangeListener() {

            public void stateChanged(ChangeEvent e) {
                joint3
                        .setText(f
                                .format((double) joint3Slider.getValue() / 100));
            }
        });

        joint4 = new JTextField("", 7);
        joint4.setEnabled(false);
        joint4.setDisabledTextColor(new Color(000000));
        joint4Slider =
                new JSlider(JSlider.HORIZONTAL, sliderMinValue4,
                        sliderMaxValue4, sliderStartValue);
        joint4Slider.addChangeListener(new ChangeListener() {

            public void stateChanged(ChangeEvent e) {
                joint4
                        .setText(f
                                .format((double) joint4Slider.getValue() / 100));
            }
        });

        joint5 = new JTextField("", 7);
        joint5.setEnabled(false);
        joint5.setDisabledTextColor(new Color(000000));
        joint5Slider =
                new JSlider(JSlider.HORIZONTAL, sliderMinValue5,
                        sliderMaxValue5, sliderStartValue);
        joint5Slider.addChangeListener(new ChangeListener() {

            public void stateChanged(ChangeEvent e) {
                joint5
                        .setText(f
                                .format((double) joint5Slider.getValue() / 100));
            }
        });

        gotopx = new JTextField("", 3);
        gotopy = new JTextField("", 3);
        gotopz = new JTextField("", 3);
        gotonx = new JTextField("", 3);
        gotony = new JTextField("", 3);
        gotonz = new JTextField("", 3);
        gotoox = new JTextField("", 3);
        gotooy = new JTextField("", 3);
        gotooz = new JTextField("", 3);
        gotoax = new JTextField("", 3);
        gotoay = new JTextField("", 3);
        gotoaz = new JTextField("", 3);
        setMovementsTxt = new JTextField("", 7);

        itWorks = new JLabel(" ");

        JButton btnJoint0 = new JButton("ok");
        btnJoint0.setActionCommand("btnJoint0");
        btnJoint0.addActionListener(this);
        JButton btnJoint1 = new JButton("ok");
        btnJoint1.setActionCommand("btnJoint1");
        btnJoint1.addActionListener(this);
        JButton btnJoint2 = new JButton("ok");
        btnJoint2.setActionCommand("btnJoint2");
        btnJoint2.addActionListener(this);
        JButton btnJoint3 = new JButton("ok");
        btnJoint3.setActionCommand("btnJoint3");
        btnJoint3.addActionListener(this);
        JButton btnJoint4 = new JButton("ok");
        btnJoint4.setActionCommand("btnJoint4");
        btnJoint4.addActionListener(this);
        JButton btnJoint5 = new JButton("ok");
        btnJoint5.setActionCommand("btnJoint5");
        btnJoint5.addActionListener(this);
        JButton btngetJoints = new JButton("list Joints");
        btngetJoints.setActionCommand("btngetJoints");
        btngetJoints.addActionListener(this);
        JButton btngoto = new JButton("goto");
        btngoto.setActionCommand("btngoto");
        btngoto.addActionListener(this);
        JButton btnListMovements = new JButton("list Movements");
        btnListMovements.setActionCommand("btnListMovements");
        btnListMovements.addActionListener(this);
        JButton btnSetMovement = new JButton("set Move");
        btnSetMovement.setActionCommand("btnSetMovement");
        btnSetMovement.addActionListener(this);
        JButton btnCloseFull = new JButton("Close Gripper Full");
        btnCloseFull.setActionCommand("btnCloseFull");
        btnCloseFull.addActionListener(this);
        JButton btnCloseHalf = new JButton("Close Gripper Half");
        btnCloseHalf.setActionCommand("btnCloseHalf");
        btnCloseHalf.addActionListener(this);
        JButton btnOpen = new JButton("Open Gripper");
        btnOpen.setActionCommand("btnOpen");
        btnOpen.addActionListener(this);

        movementBox = new JComboBox(movementsList);

        // cont, gbl, comp, x, y, width, height, weightx, weighty
        addComponent(pane, gbl, new JLabel("Set joint 0 to"), 0, 0, 3, 1, 0, 0);
        addComponent(pane, gbl, joint0Slider, 3, 0, 4, 1, 0, 0);
        addComponent(pane, gbl, joint0, 7, 0, 2, 1, 0, 0);
        addComponent(pane, gbl, btnJoint0, 9, 0, 3, 1, 0, 0);
        addComponent(pane, gbl, new JLabel("Set joint 1 to"), 0, 1, 3, 1, 0, 0);
        addComponent(pane, gbl, joint1Slider, 3, 1, 4, 1, 0, 0);
        addComponent(pane, gbl, joint1, 7, 1, 2, 1, 0, 0);
        addComponent(pane, gbl, btnJoint1, 9, 1, 3, 1, 0, 0);
        addComponent(pane, gbl, new JLabel("Set joint 2 to"), 0, 2, 3, 1, 0, 0);
        addComponent(pane, gbl, joint2Slider, 3, 2, 4, 1, 0, 0);
        addComponent(pane, gbl, joint2, 7, 2, 2, 1, 0, 0);
        addComponent(pane, gbl, btnJoint2, 9, 2, 3, 1, 0, 0);
        addComponent(pane, gbl, new JLabel("Set joint 3 to"), 0, 3, 3, 1, 0, 0);
        addComponent(pane, gbl, joint3Slider, 3, 3, 4, 1, 0, 0);
        addComponent(pane, gbl, joint3, 7, 3, 2, 1, 0, 0);
        addComponent(pane, gbl, btnJoint3, 9, 3, 3, 1, 0, 0);
        addComponent(pane, gbl, new JLabel("Set joint 4 to"), 0, 4, 3, 1, 0, 0);
        addComponent(pane, gbl, joint4Slider, 3, 4, 4, 1, 0, 0);
        addComponent(pane, gbl, joint4, 7, 4, 2, 1, 0, 0);
        addComponent(pane, gbl, btnJoint4, 9, 4, 3, 1, 0, 0);
        addComponent(pane, gbl, new JLabel("Set joint 5 to"), 0, 5, 3, 1, 0, 0);
        addComponent(pane, gbl, joint5Slider, 3, 5, 4, 1, 0, 0);
        addComponent(pane, gbl, joint5, 7, 5, 2, 1, 0, 0);
        addComponent(pane, gbl, btnJoint5, 9, 5, 3, 1, 0, 0);
        addComponent(pane, gbl, new JLabel(" "), 0, 6, 12, 1, 0, 0);
        addComponent(pane, gbl, btngetJoints, 0, 7, 12, 1, 0, 0);
        addComponent(pane, gbl, new JLabel(" "), 0, 8, 12, 1, 0, 0);
        addComponent(pane, gbl, new JLabel("Goto: "), 0, 9, 2, 1, 0, 0);
        addComponent(pane, gbl, new JLabel("P: "), 2, 9, 1, 1, 0, 0);
        addComponent(pane, gbl, gotopx, 3, 9, 2, 1, 0, 0);
        addComponent(pane, gbl, gotopy, 5, 9, 1, 1, 0, 0);
        addComponent(pane, gbl, gotopz, 6, 9, 1, 1, 0, 0);
        addComponent(pane, gbl, new JLabel("N: "), 2, 10, 1, 1, 0, 0);
        addComponent(pane, gbl, gotonx, 3, 10, 2, 1, 0, 0);
        addComponent(pane, gbl, gotony, 5, 10, 1, 1, 0, 0);
        addComponent(pane, gbl, gotonz, 6, 10, 1, 1, 0, 0);
        addComponent(pane, gbl, new JLabel("O: "), 2, 11, 1, 1, 0, 0);
        addComponent(pane, gbl, gotoox, 3, 11, 2, 1, 0, 0);
        addComponent(pane, gbl, gotooy, 5, 11, 1, 1, 0, 0);
        addComponent(pane, gbl, gotooz, 6, 11, 1, 1, 0, 0);
        addComponent(pane, gbl, new JLabel("A: "), 2, 12, 1, 1, 0, 0);
        addComponent(pane, gbl, gotoax, 3, 12, 2, 1, 0, 0);
        addComponent(pane, gbl, gotoay, 5, 12, 1, 1, 0, 0);
        addComponent(pane, gbl, gotoaz, 6, 12, 1, 1, 0, 0);
        addComponent(pane, gbl, btngoto, 8, 12, 3, 1, 0, 0);
        addComponent(pane, gbl, new JLabel(" "), 0, 13, 5, 1, 0, 0);
        addComponent(pane, gbl, movementBox, 0, 14, 8, 1, 0, 0);
        addComponent(pane, gbl, btnSetMovement, 8, 14, 2, 1, 0, 0);
        addComponent(pane, gbl, new JLabel(" "), 0, 15, 5, 1, 0, 0);
        addComponent(pane, gbl, new JLabel(" "), 0, 18, 5, 1, 0, 0);
        addComponent(pane, gbl, btnOpen, 0, 19, 4, 1, 0, 0);
        addComponent(pane, gbl, btnCloseHalf, 4, 19, 2, 1, 0, 0);
        addComponent(pane, gbl, btnCloseFull, 6, 19, 3, 1, 0, 0);
        addComponent(pane, gbl, new JLabel(" "), 0, 20, 5, 1, 0, 0);
        addComponent(pane, gbl, itWorks, 0, 21, 10, 1, 0, 0);
        addComponent(pane, gbl, new JLabel(" "), 0, 22, 5, 1, 0, 0);

        gui.pack();
        gui.setVisible(true);
    }

    private void clearFields() {
        joint0.setText("");
        joint1.setText("");
        joint2.setText("");
        joint3.setText("");
        joint4.setText("");
        joint5.setText("");
    }

    private void showJoints() {
        LinkedList<Double> joints;

        try {
            joints = poseActuatorTobi.listJoints();

            Double joint0Value = joints.get(0) * 100;
            Double joint1Value = joints.get(1) * 100;
            Double joint2Value = joints.get(2) * 100;
            Double joint3Value = joints.get(3) * 100;
            Double joint4Value = joints.get(4) * 100;
            Double joint5Value = joints.get(5) * 100;

            joint0Slider.setValue(joint0Value.intValue());
            joint0.setText(f.format(joints.get(0)));
            joint1Slider.setValue(joint1Value.intValue());
            joint1.setText(f.format(joints.get(1)));
            joint2Slider.setValue(joint2Value.intValue());
            joint2.setText(f.format(joints.get(2)));
            joint3Slider.setValue(joint3Value.intValue());
            joint3.setText(f.format(joints.get(3)));
            joint4Slider.setValue(joint4Value.intValue());
            joint4.setText(f.format(joints.get(4)));
            joint5Slider.setValue(joint5Value.intValue());
            joint5.setText(f.format(joints.get(5)));
        } catch (IOException e) {
            logger.error(e);
        }
    }

    @Override
    public void actionPerformed(ActionEvent e) {
        try {
            if (e.getActionCommand().equals("btnJoint0")) {
                poseActuatorTobi.moveJoint(0, Double.parseDouble(joint0
                        .getText()));
                clearFields();
            } else if (e.getActionCommand().equals("btnJoint1")) {
                poseActuatorTobi.moveJoint(1, Double.parseDouble(joint1
                        .getText()));
                clearFields();
            } else if (e.getActionCommand().equals("btnJoint2")) {
                poseActuatorTobi.moveJoint(2, Double.parseDouble(joint2
                        .getText()));
                clearFields();
            } else if (e.getActionCommand().equals("btnJoint3")) {
                poseActuatorTobi.moveJoint(3, Double.parseDouble(joint3
                        .getText()));
                clearFields();
            } else if (e.getActionCommand().equals("btnJoint4")) {
                poseActuatorTobi.moveJoint(4, Double.parseDouble(joint4
                        .getText()));
                clearFields();
            } else if (e.getActionCommand().equals("btnJoint5")) {
                poseActuatorTobi.moveJoint(5, Double.parseDouble(joint5
                        .getText()));
                clearFields();
            } else if (e.getActionCommand().equals("btngetJoints")) {
                clearFields();
                showJoints();
            } else if (e.getActionCommand().equals("btngoto")) {
                clearFields();
                boolean result =
                        poseActuatorTobi.goTo(Double.parseDouble(gotopx
                                .getText()), Double.parseDouble(gotopy
                                .getText()), Double.parseDouble(gotopz
                                .getText()), Double.parseDouble(gotonx
                                .getText()), Double.parseDouble(gotony
                                .getText()), Double.parseDouble(gotonz
                                .getText()), Double.parseDouble(gotoax
                                .getText()), Double.parseDouble(gotoay
                                .getText()), Double.parseDouble(gotoaz
                                .getText()), Double.parseDouble(gotoox
                                .getText()), Double.parseDouble(gotooy
                                .getText()), Double.parseDouble(gotooz
                                .getText()));
                if (result) {
                    itWorks.setText("Position reached");
                    showJoints();
                } else {
                    itWorks.setText("Could not reach position!");
                }
            } else if (e.getActionCommand().equals("btnSetMovement")) {
                clearFields();
                poseActuatorTobi.setMovements((String) movementBox
                        .getSelectedItem());
            } else if (e.getActionCommand().equals("btnOpen")) {
                clearFields();
                poseActuatorTobi.openGripper();
                itWorks.setText("Open Gripper");
            } else if (e.getActionCommand().equals("btnCloseFull")) {
                clearFields();
                poseActuatorTobi.closeGripper();
                itWorks.setText("Close Gripper full");
            } else if (e.getActionCommand().equals("btnCloseHalf")) {
                clearFields();
                poseActuatorTobi.closeGripper();
                itWorks.setText("Close Gripper half");
            }
        } catch (NumberFormatException e1) {
            logger.error(e1);
        } catch (IOException e1) {
            logger.error(e1);
        }

    }

    public static void main(String[] args) {

        new ArmControlRemote();
    }

}
