package de.unibi.airobots.bonsai.examples;

import java.awt.image.BufferedImage;
import java.io.IOException;
import java.net.URI;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Set;

import javax.swing.ImageIcon;
import javax.swing.JFrame;
import javax.swing.JLabel;

import org.apache.log4j.BasicConfigurator;
import org.apache.log4j.Level;
import org.apache.log4j.Logger;

import de.unibi.airobots.bonsai.core.BonsaiManager;
import de.unibi.airobots.bonsai.core.Sensor;
import de.unibi.airobots.bonsai.core.SensorListener;
import de.unibi.airobots.bonsai.core.configuration.ConfigurationParser;
import de.unibi.airobots.bonsai.core.configuration.ConfiguredActuator;
import de.unibi.airobots.bonsai.core.configuration.ConfiguredCoreObjectFactory;
import de.unibi.airobots.bonsai.core.configuration.ConfiguredSensor;
import de.unibi.airobots.bonsai.sensors.data.ImageData;
import de.unibi.airobots.bonsai.xcf.XcfFactory;
import de.unibi.airobots.bonsai.xcf.configuration.ConfiguredXcfCameraSensor;
import de.unibi.airobots.bonsai.xcf.sensors.XcfCameraSensor;

/**
 * A simple test program for the camera sensor.
 * 
 * @author jwienke
 */
public class CameraSensorTest implements Runnable {

    /**
     * A simple example for using the CameraSensor.
     * 
     * @param args
     *            command line arguments
     */
    public static void main(String[] args) {
        BasicConfigurator.configure();
        Logger.getRootLogger().setLevel(Level.INFO);
        new CameraSensorTest().run();
    }

    /**
     * Constructor.
     */
    public CameraSensorTest() {

    }

    @Override
    public void run() {

        try {

            BonsaiManager manager = BonsaiManager.getInstance();
            manager.setParser(new ConfigurationParser() {

                @Override
                public Set<ConfiguredActuator> getConfiguredActuators()
                        throws IllegalStateException {
                    return new HashSet<ConfiguredActuator>();
                }

                @Override
                public Set<ConfiguredCoreObjectFactory> getConfiguredFactories()
                        throws IllegalStateException {

                    HashSet<ConfiguredCoreObjectFactory> factories =
                            new HashSet<ConfiguredCoreObjectFactory>();

                    ConfiguredCoreObjectFactory xcfFactory =
                            new ConfiguredCoreObjectFactory();
                    xcfFactory.setFactoryClassName(XcfFactory.class.getName());
                    xcfFactory.setFactroyOptions(new HashMap<String, String>());
                    factories.add(xcfFactory);

                    return factories;

                }

                @Override
                public Set<ConfiguredSensor> getConfiguredSensors()
                        throws IllegalStateException {

                    HashSet<ConfiguredSensor> sensors =
                            new HashSet<ConfiguredSensor>();

                    ConfiguredSensor cameraSensor = new ConfiguredSensor();
                    cameraSensor.setDataType(ImageData.class.getName());
                    cameraSensor
                            .setFactoryClassName(XcfFactory.class.getName());
                    cameraSensor.setKey("CameraSensor");
                    HashMap<String, String> camOptions =
                            new HashMap<String, String>();
                    camOptions.put(
                            ConfiguredXcfCameraSensor.OPTION_PUBLISHER_NAME,
                            "RobbyCam");
                    cameraSensor.setOptions(camOptions);
                    cameraSensor.setSensorClassName(XcfCameraSensor.class
                            .getName());
                    sensors.add(cameraSensor);

                    return sensors;

                }

                @Override
                public void parseFile(URI configurationUrl) throws IOException,
                        ParseException, IllegalStateException {
                    // ignore this
                }

            });
            manager.configure(new URI("file://foo"));

            // We create a new JFrame
            final JFrame frame = new JFrame("Camera Sensor Image");
            frame.setDefaultCloseOperation(JFrame.DISPOSE_ON_CLOSE);
            frame.setResizable(false);

            final JLabel leLabel = new JLabel();
            frame.add(leLabel);
            Sensor<ImageData> camSens =
                    manager.createSensor("CameraSensor", ImageData.class);
            frame.pack();
            frame.setVisible(true);

            // synchronous variant
            // while (true) {
            //
            // System.out.println("New loop");
            //
            // // readLast() delivers an ImageData from the XCFExport which can
            // // create a BufferedImage
            // ImageData imgdata = camSens.readLast();
            // BufferedImage img = imgdata.createBufferedImage();
            //
            // frame.setSize(img.getWidth(), img.getHeight());
            // leLabel.setIcon(new ImageIcon(img));
            // frame.repaint();
            //
            // }

            // asynchronous variant
            camSens.addSensorListener(new SensorListener<ImageData>() {

                @Override
                public void newDataAvailable(ImageData newData) {
                    BufferedImage img = newData.createBufferedImage();

                    frame.setSize(img.getWidth(), img.getHeight());
                    leLabel.setIcon(new ImageIcon(img));
                    frame.repaint();
                }

            });

            // CHECKSTYLE:OFF
        } catch (Exception e) {
            e.printStackTrace();
            // CHECKSTYLE:ON
            System.exit(1);
        }

    }

}
