package de.unibi.airobots.bonsai.examples;

import net.sf.xcf.RemoteServer;
import net.sf.xcf.UserException;
import net.sf.xcf.XcfManager;
import net.sf.xcf.transport.XOPData;

/**
 * Simple test to see what happens if the connection to a remote server dies.
 * 
 * @author jwienke
 */
public class ServerDisconnectTest {

    /**
     * Main method.
     * 
     * @param args
     *            server name, method to call
     */
    public static void main(String[] args) {

        if (args.length != 2) {
            System.err.println("Required arguments: "
                    + "[server name] [method to call]");
            System.exit(1);
        }

        try {

            XcfManager xcfManager = XcfManager.createXcfManager();

            RemoteServer server = xcfManager.createRemoteServer(args[0]);

            System.out.println("First call to method");
            XOPData data = XOPData.fromString("<test/>");
            try {
                server.callMethod(args[1], data);
            } catch (UserException e) {
                System.out.println("Received " + e.getClass().getSimpleName()
                        + ", but that may be ok for testing.");
                e.printStackTrace();
            }

            System.out.println("--- Called method, "
                    + "now waiting 5 seconds before next call ---");

            // CHECKSTYLE:OFF
            Thread.sleep(5000);
            // CHECKSTYLE:ON

            System.out.println("Second call to method");
            try {
                server.callMethod(args[1], data);
            } catch (UserException e) {
                System.out.println("Received " + e.getClass().getSimpleName()
                        + ", but that may be ok for testing.");
                e.printStackTrace();
            }

            // CHECKSTYLE:OFF
        } catch (Exception e) {
            // CHECKSTYLE:ON
            System.err.println("Unexpected error:");
            e.printStackTrace();
            System.exit(1);
        }

    }

}
