package de.unibi.airobots.bonsai.examples;

import java.util.HashMap;
import java.util.Map;

import org.apache.log4j.BasicConfigurator;

import net.sf.xcf.MethodNotFoundException;
import net.sf.xcf.RemoteServer;
import net.sf.xcf.UserException;
import net.sf.xcf.XcfManager;
import net.sf.xcf.transport.XOPData;
import net.sf.xcf.xml.SyntaxException;
import de.unibi.airobots.bonsai.core.Actuator;
import de.unibi.airobots.bonsai.xcf.RemoteServerReconnectable;
import de.unibi.airobots.bonsai.xcf.RemoteServerReconnectableManager;

public class TestServerCaller {

    private static class TestActuator implements Actuator,
            RemoteServerReconnectable {

        private RemoteServer server = null;

        /**
         * {@inheritDoc}
         */
        @Override
        public Map<String, ErrorHandlingPolicy> getRemoteServerNames() {
            Map<String, ErrorHandlingPolicy> servers =
                    new HashMap<String, ErrorHandlingPolicy>();
            servers.put(TestServer.SERVER_NAME,
                    ErrorHandlingPolicy.SELF_HEALING);
            return servers;
        }

        /**
         * {@inheritDoc}
         */
        @Override
        public void resetRemoteServer(RemoteServer remoteServer) {
            this.server = remoteServer;
        }

        public void callMethod() {

            if (server == null) {
                throw new IllegalStateException();
            }

            try {
                System.out.println("Calling method");
                XOPData result =
                        server.callMethod(TestServer.METHOD_NAME, XOPData
                                .fromString("<callIt/>"));
                System.out.println("result received: " + result);
            } catch (MethodNotFoundException e) {
                e.printStackTrace();
            } catch (UserException e) {
                e.printStackTrace();
            } catch (SyntaxException e) {
                e.printStackTrace();
            }

        }

    }

    public static void main(String[] args) {

        try {

            BasicConfigurator.configure();

            XcfManager xcfManager = XcfManager.createXcfManager();
            RemoteServerReconnectableManager manager =
                    new RemoteServerReconnectableManager(xcfManager);
            manager.initializeXcfType(TestServer.SERVER_NAME);

            TestActuator actuator = new TestActuator();
            manager.initializeAndManage(actuator);

            // wait until the actuator is probably initialized with the
            // server...
            while (true) {

                actuator.callMethod();
                System.out.println("Waiting 2 seconds");
                Thread.sleep(2000);

            }

        } catch (Exception e) {
            e.printStackTrace();
            System.exit(1);
        }

    }

}
