package de.unibi.airobots.bonsai.xcf;

import net.sf.xcf.Subscriber;

/**
 * Interface for objects that can reconnect to a subscriber while running if an
 * external component decided that a new subscriber instance must be used.
 * 
 * After constructing an object implementing this interface, the
 * {@link #resetSubscriber(Subscriber)} must be called once to initialize the
 * object on the first subscriber.
 * 
 * @author jwienke
 */
public interface SubscriberReconnectable {

    /**
     * Returns the name of the subscriber to work on.
     * 
     * @return subscriber name
     */
    String getSubscriberName();

    /**
     * Reconnect to or initialize this object with the given subscriber.
     * 
     * @param subscriber
     *            (new) subscriber to use from now on
     */
    void resetSubscriber(Subscriber subscriber);

}
