package de.unibi.airobots.bonsai.xcf.actuators;

import java.io.IOException;
import java.util.Calendar;
import java.util.HashMap;
import java.util.Map;

import net.sf.xcf.MethodNotFoundException;
import net.sf.xcf.RemoteServer;
import net.sf.xcf.UserException;
import net.sf.xcf.transport.XOPData;
import nu.xom.Attribute;
import nu.xom.Document;
import nu.xom.Element;

import org.apache.log4j.Logger;

import de.unibi.airobots.bonsai.actuators.EnterArenaAndLocalizeActuator;
import de.unibi.airobots.bonsai.xcf.RemoteServerReconnectable;

/**
 * Arena entering and localization actuator
 * 
 * @author ltwardon
 */
public class XcfEnterArenaAndLocalizeActuator implements
        EnterArenaAndLocalizeActuator, RemoteServerReconnectable {

    private Logger logger = Logger.getLogger(getClass());

    private String remoteServerName = "";
    private Object remoteServerLock = new Object();
    private RemoteServer remoteServer = null;

    /**
     * Constructor.
     * 
     * @param remoteServerName
     *            name of the arena entering and localization remote server
     */
    public XcfEnterArenaAndLocalizeActuator(String remoteServerName) {
        this.remoteServerName = remoteServerName;
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public Map<String, ErrorHandlingPolicy> getRemoteServerNames() {
        Map<String, ErrorHandlingPolicy> servers =
                new HashMap<String, ErrorHandlingPolicy>();
        servers.put(remoteServerName, ErrorHandlingPolicy.SELF_HEALING);
        return servers;
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public void resetRemoteServer(RemoteServer remoteServer) {
        assert remoteServer.getServerName().equals(remoteServerName);
        synchronized (remoteServerLock) {
            this.remoteServer = remoteServer;
        }
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public void enterArenaAndLocalize(double meters) throws IOException {

        synchronized (remoteServerLock) {
            if (remoteServer == null) {
                throw new IllegalStateException("No remote server set.");
            }
        }

        Element elemLocalize = new Element("localize");
        Element elemMeters = new Element("meters");
        elemMeters.appendChild(Double.toString(meters));
        Element elemPrior = new Element("priorKnowledge");
        Attribute x = new Attribute("x", "0.0");
        Attribute y = new Attribute("y", "0.0");
        Attribute div = new Attribute("div", "10.0");
        elemPrior.addAttribute(x);
        elemPrior.addAttribute(y);
        elemPrior.addAttribute(div);
        elemLocalize.appendChild(elemMeters);
        elemLocalize.appendChild(elemPrior);
        Document doc = new Document(elemLocalize);
        try {
            synchronized (remoteServerLock) {
                remoteServer.callMethod("enterArenaAndLocalize", new XOPData(
                        doc));
            }
        } catch (MethodNotFoundException e) {
            logger.error(e);
            throw new IOException(e);
        } catch (UserException e) {
            logger.error(e);
            throw new IOException(e);
        }

    }
    
    /**
     * {@inheritDoc}
     */
    @Override
    public void enterArenaAndLocalize(double meters, double priorKnowledgeX, double priorKnowledgeY, double priorKnowledgeDiv) throws IOException {
        
        synchronized (remoteServerLock) {
            if (remoteServer == null) {
                throw new IllegalStateException("No remote server set.");
            }
        }

        Element elemLocalize = new Element("localize");
        Element elemMeters = new Element("meters");
        elemMeters.appendChild(Double.toString(meters));
        Element elemPrior = new Element("priorKnowledge");
        Attribute x = new Attribute("x", Double.toString(priorKnowledgeX));
        Attribute y = new Attribute("y", Double.toString(priorKnowledgeY));
        Attribute div = new Attribute("div", Double.toString(priorKnowledgeDiv));
        elemPrior.addAttribute(x);
        elemPrior.addAttribute(y);
        elemPrior.addAttribute(div);
        elemLocalize.appendChild(elemMeters);
        elemLocalize.appendChild(elemPrior);
        Document doc = new Document(elemLocalize);
        try {
            synchronized (remoteServerLock) {
                remoteServer.callMethod("enterArenaAndLocalize", new XOPData(
                        doc));
            }
        } catch (MethodNotFoundException e) {
            logger.error(e);
            throw new IOException(e);
        } catch (UserException e) {
            logger.error(e);
            throw new IOException(e);
        }
        
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public void waitUntilLocalizationFinished() throws IOException {

        synchronized (remoteServerLock) {
            if (remoteServer == null) {
                throw new IllegalStateException("No remote server set.");
            }
        }

        boolean locFinished = false;
        XOPData input = new XOPData();
        XOPData result = new XOPData();

        long tsStart = Calendar.getInstance().getTimeInMillis();

        while (!locFinished) {
            
            logger.debug("localizing...");

            try {
                synchronized (remoteServerLock) {
                    result =
                            remoteServer.callMethod("localizationFinished",
                                    input);
                    if (result.getDocument().getRootElement().getValue()
                            .equals("yes")) {
                        locFinished = true;
                    }
                }
                Thread.sleep(100);
            } catch (InterruptedException e) {
                logger.error(e);
                throw new IOException(e);
            } catch (MethodNotFoundException e) {
                logger.error(e);
                throw new IOException(e);
            } catch (UserException e) {
                logger.error(e);
                throw new IOException(e);
            }

            long ts = Calendar.getInstance().getTimeInMillis();
        }

    }

}
