package de.unibi.airobots.bonsai.xcf.actuators;

import java.io.IOException;

import net.sf.xcf.ActiveMemory;
import net.sf.xcf.memory.DocumentNotFoundException;
import net.sf.xcf.memory.MemoryException;
import net.sf.xcf.transport.XOPData;
import net.sf.xcf.xml.XPath;
import nu.xom.Attribute;
import nu.xom.Document;
import nu.xom.Element;

import org.apache.log4j.Logger;

import de.unibi.airobots.bonsai.actuators.RobotStateActuator;
import de.unibi.airobots.bonsai.xcf.ActiveMemoryReconnectable;
import de.unibi.airobots.btl.data.RobotState;
import de.unibi.airobots.btl.data.Type.NoSourceDocumentException;

/**
 * @author ltwardon
 */
public class XcfRobotStateActuator implements RobotStateActuator,
        ActiveMemoryReconnectable {

    private Logger logger = Logger.getLogger(getClass());

    private String memoryName = "";
    private Object memoryLock = new Object();
    private ActiveMemory memory = null;
    private boolean isInserted = false;
    
    /**
     * Xml namespace for active memory documents.
     */
    public static final String NAMESPACE_MEMORY =
            "http://purl.org/ActiveMemory";

    /**
     * Xml namespace for dbxml.
     */
    public static final String NAMESPACE_DBXML =
            "http://www.sleepycat.com/2002/dbxml";

    /**
     * Creates a new instance of {@link XcfRobotStateActuator}.
     * 
     * @param memoryName
     *            Name of the memory to connect to.
     */
    public XcfRobotStateActuator(String memoryName) {
        this.memoryName = memoryName;
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public String getActiveMemoryName() {
        return memoryName;
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public void resetActiveMemory(ActiveMemory activeMemory) {
        synchronized (memoryLock) {
            this.memory = activeMemory;
        }
    }

    @Override
    public void sendState(RobotState state) throws NoSourceDocumentException, IOException {

        if (memory == null) {
            logger.error("Memory is not reachable");
        }
        try {
            if(isInserted) {
                Element source = new Element(state.getSourceDocument());
                state.fillInto(source);
                memory.replace(new XOPData(new Document(source)));
            }
            else {
                memory.insert(new XOPData(state.toDocument(RobotState.class)));
                isInserted = true;
            }
        } catch (DocumentNotFoundException e) {
            logger.error("Document not found in memory");
            logger.debug(e);
            throw new IOException(e);
        } catch (MemoryException e) {
            logger.error("Memory is not reachable");
            logger.debug(e);
            throw new IOException(e);
        }
    }

}
