package de.unibi.airobots.bonsai.xcf.configuration;

import java.util.Map;

import de.unibi.airobots.bonsai.core.Actuator;
import de.unibi.airobots.bonsai.core.CoreObjectFactory.ActuatorToConfigure;
import de.unibi.airobots.bonsai.xcf.actuators.XcfHwaActuator;

/**
 * Configuration for {@link XcfHwaActuator}.
 * <p>
 * Available options:
 * <ul>
 * <li>{@link ConfiguredXcfActuator#OPTION_SERVER_NAME} (required): name of the
 * configuration server for the hwa</li>
 * <li>{@value #OPTION_MEMORY_NAME} (required): name of the memory for inserting
 * persons to look at</li>
 * </ul>
 * 
 * @author jwienke
 */
public class ConfiguredXcfHwaActuator extends ConfiguredXcfActuator {

    /**
     * Name of the scene memory used to control the HWA.
     */
    public static final String OPTION_MEMORY_NAME = "sceneMemory";

    /**
     * Name of the memory to insert persons to look at.
     */
    private String memoryName = "";

    /**
     * Checks if the actuator that can be produced is suitable for the requested
     * configuration.
     * 
     * @param actuatorToConfigure
     *            requested configuration
     * @return <code>true</code> if suitable
     */
    public static boolean isSuitableFor(ActuatorToConfigure actuatorToConfigure) {
        return actuatorToConfigure.getActuatorClass().equals(
                XcfHwaActuator.class);
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public void parseOptions(Map<String, String> options)
            throws IllegalArgumentException {

        if (!options.containsKey(OPTION_MEMORY_NAME)) {
            throw new IllegalArgumentException("Missing option '"
                    + OPTION_MEMORY_NAME + "'");
        }

        this.memoryName = options.get(OPTION_MEMORY_NAME);

    }

    /**
     * {@inheritDoc}
     */
    @Override
    public Actuator createInstance() {
        return new XcfHwaActuator(getServerName(), memoryName);
    }

}
