package de.unibi.airobots.bonsai.xcf.sensors;

import java.io.IOException;

import net.sf.xcf.transport.Attachment;
import net.sf.xcf.transport.SerializeException;
import net.sf.xcf.transport.XOPData;

import org.apache.log4j.Logger;

import de.unibi.airobots.btl.data.BinarySlamMap;

/**
 * The SlamSensor receives the current SlamData provided by the robot.
 * 
 * @author sebschne
 * @author dklotz
 * @author jwienke
 */
public class XcfBinarySlamSensor extends
        XcfBtlPublisherSensor<BinarySlamMap> {

    private Logger logger = Logger.getLogger(getClass());

    /**
     * Constructor.
     * 
     * @param subscriberName
     *            name of the publisher providing the slam data
     */
    public XcfBinarySlamSensor(String subscriberName) {
        super(BinarySlamMap.class, subscriberName);
    }

    /**
     * Extract slam map attachment.
     * 
     * {@inheritDoc}
     */
    @Override
    protected void postProcessType(XOPData data, BinarySlamMap extractedType)
            throws IOException {

        try {
            Attachment mapPacket = data.getAttachment(extractedType.getUri());
            extractedType.setSlamMap(mapPacket.getAsFloatArray());
        } catch (SerializeException e) {
            logger.error("Error decoding slam map attachment.", e);
            throw new IOException(e);
        }

    }

}
